#define FIND_H

/****************************************************************

    Copyright (c) 1996-1997 Fred Pesch

    This program is free software; you can redistribute it 
    and/or modify it under the terms of the GNU General 
    Public License as published by the Free Software 
    Foundation (version 2 of the License).

    This program is distributed in the hope that it will 
    be useful, but WITHOUT ANY WARRANTY; without even the 
    implied warranty of MERCHANTABILITY or FITNESS FOR A 
    PARTICULAR PURPOSE.  See the GNU General Public License 
    for more details.

    For a copy of the GNU General Public License, write to the 
    Free Software Foundation, Inc., 675 Mass Ave, Cambridge, 
    MA 02139, USA.

****************************************************************/

/* includes */

#include <Xm/Xm.h>
#include <Xm/MainW.h>
#include <Xm/PushB.h>
#include <Xm/Text.h>
#include <Xm/Form.h>
#include <Xm/ScrolledW.h>
#include <Xm/Separator.h>
#include <Xm/List.h>
#include <Xm/Frame.h>
#include <Xm/Label.h>
#include <Xm/TextF.h>
#include <Xm/ToggleB.h>
#include <Xm/RowColumn.h>
#include <Xm/CascadeB.h>

#include <stdlib.h>
#include <signal.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/wait.h>
#include <sys/types.h>

#include "interface.h"
#include "error_handler.h"
#include "select.h"

#ifdef OFFIX_DND
  #include "offix_dnd.h"
#endif

extern int GoToDirRefreshRoutine(char *destdir);

/* constants */

#define FIND_BUFFER_SIZE_MAX 65536

/* global variables */

Widget W_FindDialog=NULL;
Widget W_FindBasicCriteria=NULL;
Widget W_FindAdvancedCriteria=NULL;
Widget W_FindList=NULL;

XtInputId FindInputId;

int FindChildExist=0;

struct sigaction FindChildAction;
struct sigaction FindOldAction;

pid_t FindChildPID;

int FindOptionSize1=1;
int FindOptionSize2=1;
int FindOptionMtime=1;
int FindOptionCtime=1;
int FindOptionAtime=1;

char *FindResultBuffer;
int FindResultBufferSize;

/* Misc functions */

void ConvertFindBuffer(void)
{
  char lastline[2048];
  XmString xentrystr;
  int i,j;

  i=0;
  while(FindResultBuffer[i]!=(char) 0) {
    j=0;
    while(FindResultBuffer[i]!='\n' && FindResultBuffer[i]!=(char)0)
      lastline[j++]=FindResultBuffer[i++];
    if(FindResultBuffer[i]=='\n') {
      i++;
      lastline[j]=(char) 0;
      xentrystr = XmStringCreateSimple(lastline);
      XmListAddItemUnselected(W_FindList, xentrystr, 0);
      XmStringFree(xentrystr);
    }
  }

  XtFree(FindResultBuffer);
  FindResultBufferSize=0;
}

void GetFindResult( XtPointer clientData, int *fid, XtInputId *id)
{
  char buf[BUFSIZ+1];
  int nbbytes;

  nbbytes=read(*fid, buf, BUFSIZ);

  if(nbbytes>0) {
    buf[nbbytes]=(char)0;
    FindResultBufferSize+=strlen(buf);
    if(FindResultBufferSize< FIND_BUFFER_SIZE_MAX)
      strcat(FindResultBuffer, buf); /* else show a message */
  }    
  
}

void FindChildHandler(int sig)
{
  int status;
  int result;
  Widget w_temp;
  int count;
  char message[64];
  XmString xmessage;

  if(!FindChildExist) return;

  waitpid(FindChildPID, &status, 0);

  result=sigaction(SIGCHLD, &FindOldAction, NULL);
  /*  if(result==-1) FatalError("shell.c", "ShellChildHandler","Unable to install the default child signal handler");*/

  XtRemoveInput(FindInputId);
  FindChildExist=0;

  ConvertFindBuffer();

  XtVaGetValues(W_FindList, XmNitemCount, &count, NULL);
  sprintf(message, "Files Found ( %d matches )",count);
  xmessage = XmStringCreateSimple(message);
  w_temp=XtNameToWidget(W_FindDialog, "*FindLabel2");
  XtVaSetValues(w_temp, XmNlabelString, xmessage, NULL);
  XmStringFree(xmessage);

  w_temp=XtNameToWidget(W_FindDialog,"*FindStopButton");
  XtVaSetValues(w_temp, XmNsensitive, False, NULL);  
}

void FindExecCommand(char *cmdstr)
{
  pid_t child;
  int result;
  char *arglist[256];
  int countarg;
  int pipe_to_parent[2];
  int resultex;
  Widget w_temp;

  FindResultBuffer=XtMalloc(sizeof(char)*FIND_BUFFER_SIZE_MAX);
  if(FindResultBuffer==NULL) return; /* show a message */

  FindResultBufferSize=1;

  FindChildExist=1;

  strcpy(FindResultBuffer, "");

  XmListDeleteAllItems(W_FindList);
  /*  XtVaSetValues(W_FindList, XmNitems, NULL, XmNitemCount, 0, NULL);*/
  
  FindInputId=XtAppAddInput(AppContext, fileno(stdin), (XtPointer) XtInputReadMask, GetFindResult, NULL);

  w_temp=XtNameToWidget(W_FindDialog,"*FindStopButton");
  XtVaSetValues(w_temp, XmNsensitive, True, NULL);

  pipe(pipe_to_parent);

  /* first install the child handler */
  FindChildAction.sa_handler=FindChildHandler;
  result=sigaction(SIGCHLD, &FindChildAction, &FindOldAction);
  /*if(result==-1) FatalError("shell.c", "ShellExecCommand","Unable to install signal handler");*/
  child=fork();
  /*  if(child==-1) FatalError("shell.c", "ShellExecCommand","unable to fork()");*/
  if(child==0) {
    countarg=0;
    arglist[countarg]=strtok(cmdstr," ");
    while(arglist[countarg]!=NULL) {
      countarg++;
      arglist[countarg]=strtok(NULL, " ");
    } 

    close(pipe_to_parent[0]);
    result=dup2(pipe_to_parent[1],fileno(stdout));
    /*    result=dup2(pipe_to_parent[1],fileno(stderr));*/
    resultex=execvp(arglist[0], arglist);            
    if(resultex==-1) {
      sleep(3);
      exit(1);
    } 
  } 

  FindChildPID=child;

  result=dup2(pipe_to_parent[0],fileno(stdin));
  close(pipe_to_parent[1]);
  
}


/* Callbacks */

/* command callbacks */

void SizeOption1CB(Widget w_button, XtPointer app_data, XtPointer widget_data)
{
  XmRowColumnCallbackStruct *infos;
  Widget w_temp;

  infos =  (XmRowColumnCallbackStruct *) widget_data;

  w_temp= XtNameToWidget(W_FindDialog, "*FindSizeExact");
  if(w_temp==infos->widget) {
    FindOptionSize1=1;
    return;
  }
  w_temp= XtNameToWidget(W_FindDialog, "*FindSizeLess");
  if(w_temp==infos->widget) {
    FindOptionSize1=2;
    return;
  }
  w_temp= XtNameToWidget(W_FindDialog, "*FindSizeMore");
  if(w_temp==infos->widget) {
    FindOptionSize1=3;
    return;
  }
}

void SizeOption2CB(Widget w_button, XtPointer app_data, XtPointer widget_data)
{
  XmRowColumnCallbackStruct *infos;
  Widget w_temp;

  infos =  (XmRowColumnCallbackStruct *) widget_data;

  w_temp= XtNameToWidget(W_FindDialog, "*FindSizeBytes");
  if(w_temp==infos->widget) {
    FindOptionSize2=1;
    return;
  }
  w_temp= XtNameToWidget(W_FindDialog, "*FindSizeKilos");
  if(w_temp==infos->widget) {
    FindOptionSize2=2;
    return;
  }
  w_temp= XtNameToWidget(W_FindDialog, "*FindSizeBlocks");
  if(w_temp==infos->widget) {
    FindOptionSize2=3;
    return;
  }
  w_temp= XtNameToWidget(W_FindDialog, "*FindSizeWords");
  if(w_temp==infos->widget) {
    FindOptionSize2=4;
    return;
  }
} 

void MtimeOptionCB(Widget w_button, XtPointer app_data, XtPointer widget_data)
{
  XmRowColumnCallbackStruct *infos;
  Widget w_temp;

  infos =  (XmRowColumnCallbackStruct *) widget_data;

  w_temp= XtNameToWidget(W_FindDialog, "*FindMtimeExact");
  if(w_temp==infos->widget) {
    FindOptionMtime=1;
    return;
  }
  w_temp= XtNameToWidget(W_FindDialog, "*FindMtimeLess");
  if(w_temp==infos->widget) {
    FindOptionMtime=2;
    return;
  }
  w_temp= XtNameToWidget(W_FindDialog, "*FindMtimeMore");
  if(w_temp==infos->widget) {
    FindOptionMtime=3;
    return;
  }
}

void CtimeOptionCB(Widget w_button, XtPointer app_data, XtPointer widget_data)
{
  XmRowColumnCallbackStruct *infos;
  Widget w_temp;

  infos =  (XmRowColumnCallbackStruct *) widget_data;

  w_temp= XtNameToWidget(W_FindDialog, "*FindCtimeExact");
  if(w_temp==infos->widget) {
    FindOptionCtime=1;
    return;
  }
  w_temp= XtNameToWidget(W_FindDialog, "*FindCtimeLess");
  if(w_temp==infos->widget) {
    FindOptionCtime=2;
    return;
  }
  w_temp= XtNameToWidget(W_FindDialog, "*FindCtimeMore");
  if(w_temp==infos->widget) {
    FindOptionCtime=3;
    return;
  }
}
void AtimeOptionCB(Widget w_button, XtPointer app_data, XtPointer widget_data)
{
  XmRowColumnCallbackStruct *infos;
  Widget w_temp;

  infos =  (XmRowColumnCallbackStruct *) widget_data;

  w_temp= XtNameToWidget(W_FindDialog, "*FindAtimeExact");
  if(w_temp==infos->widget) {
    FindOptionAtime=1;
    return;
  }
  w_temp= XtNameToWidget(W_FindDialog, "*FindAtimeLess");
  if(w_temp==infos->widget) {
    FindOptionAtime=2;
    return;
  }
  w_temp= XtNameToWidget(W_FindDialog, "*FindAtimeMore");
  if(w_temp==infos->widget) {
    FindOptionAtime=3;
    return;
  }
}


void FindExecButtonCB(Widget w_button, XtPointer app_data, XtPointer widget_data)
{
  char *command;
  int dummy;
  char *retstr;
  Widget w_temp;
  char tempstr[4];

  if(FindChildExist) {
    if(ConfirmDialog("Previous search still running", "Do you want to abort the previous search ?")) {
      if(FindChildExist) {
	kill(FindChildPID, SIGKILL);
	sleep(3);
      }
    
    } else return;
  } 
  
  command = XtMalloc(sizeof(char)*8192); /* I think this limit is enough */
                                     
  strcpy(command, "find ");

  /* get the find path */

  w_temp = XtNameToWidget(W_FindDialog, "*FindDirText");
  retstr = XmTextFieldGetString(w_temp);

  strcat(command, retstr);
  strcat(command, " ");

  /* get the options */

  /* name */

  w_temp = XtNameToWidget(W_FindDialog, "*FindBasicNameToggle");
  if(XmToggleButtonGetState(w_temp)) {
    w_temp = XtNameToWidget(W_FindDialog, "*FindBasicNameText");
    retstr = XmTextFieldGetString(w_temp);
    strcat(command, "-name ");
    strcat(command, retstr);
    strcat(command, " ");
  }

  /* type */

  w_temp = XtNameToWidget(W_FindDialog, "*FindBasicTypeToggle");
  if(XmToggleButtonGetState(w_temp)) {
    strcat(command, "( ");
    dummy=0;
    w_temp = XtNameToWidget(W_FindDialog, "*FindTypeRegular");
    if(XmToggleButtonGetState(w_temp)) {
      strcat(command, "-type f ");
      dummy++;
    }
    w_temp = XtNameToWidget(W_FindDialog, "*FindTypeDir");
    if(XmToggleButtonGetState(w_temp)) {
      if(dummy!=0) strcat(command,"-o ");
      strcat(command, "-type d ");
      dummy++;
    }
    w_temp = XtNameToWidget(W_FindDialog, "*FindTypeSLink");
    if(XmToggleButtonGetState(w_temp)) {
      if(dummy!=0) strcat(command,"-o ");
      strcat(command, "-type l ");
      dummy++;
    }
    w_temp = XtNameToWidget(W_FindDialog, "*FindTypeBlock");
    if(XmToggleButtonGetState(w_temp)) {
      if(dummy!=0) strcat(command,"-o ");
      strcat(command, "-type b ");
      dummy++;
    }
    w_temp = XtNameToWidget(W_FindDialog, "*FindTypeChar");
    if(XmToggleButtonGetState(w_temp)) {
      if(dummy!=0) strcat(command,"-o ");
      strcat(command, "-type c ");
      dummy++;
    }
    w_temp = XtNameToWidget(W_FindDialog, "*FindTypeFifo");
    if(XmToggleButtonGetState(w_temp)) {
      if(dummy!=0) strcat(command,"-o ");
      strcat(command, "-type p ");
      dummy++;
    }
    w_temp = XtNameToWidget(W_FindDialog, "*FindTypeSocket");
    if(XmToggleButtonGetState(w_temp)) {
      if(dummy!=0) strcat(command,"-o ");
      strcat(command, "-type s ");
      dummy++;
    }        
    strcat(command, ") ");
    if(dummy==0) command[strlen(command)-4]=(char)0;
  }


  /* size */

  w_temp = XtNameToWidget(W_FindDialog, "*FindBasicSizeToggle");
  if(XmToggleButtonGetState(w_temp)) {
    strcat(command,"-size ");
    if(FindOptionSize1==2) strcat(command,"-");
    else if(FindOptionSize1==3) strcat(command,"+");
    w_temp = XtNameToWidget(W_FindDialog, "*FindSizeValue");
    retstr = XmTextFieldGetString(w_temp);
    strcat(command, retstr);
    if(FindOptionSize2==1) strcat(command,"c ");
    else if(FindOptionSize2==2) strcat(command, "k ");
    else if(FindOptionSize2==3) strcat(command, "b ");
    else if(FindOptionSize2==4) strcat(command, "w ");

  }

  /* user */

  w_temp = XtNameToWidget(W_FindDialog, "*FindBasicUserToggle");
  if(XmToggleButtonGetState(w_temp)) {
    w_temp = XtNameToWidget(W_FindDialog, "*FindBasicUserText");
    retstr = XmTextFieldGetString(w_temp);
    strcat(command, "-user ");
    strcat(command, retstr);
    strcat(command, " ");
  }

  /* group */

  w_temp = XtNameToWidget(W_FindDialog, "*FindBasicGroupToggle");
  if(XmToggleButtonGetState(w_temp)) {
    w_temp = XtNameToWidget(W_FindDialog, "*FindBasicGroupText");
    retstr = XmTextFieldGetString(w_temp);
    strcat(command, "-group ");
    strcat(command, retstr);
    strcat(command, " ");
  }

  /* one filesystem */

  w_temp = XtNameToWidget(W_FindDialog, "*FindBasicOneFsToggle");
  if(XmToggleButtonGetState(w_temp))
    strcat(command, "-xdev ");

  /* mtime */

  w_temp = XtNameToWidget(W_FindDialog, "*FindAdvancedMtimeToggle");
  if(XmToggleButtonGetState(w_temp)) {
    strcat(command,"-mtime ");
    if(FindOptionMtime==2) strcat(command,"-");
    else if(FindOptionMtime==3) strcat(command,"+");
    w_temp = XtNameToWidget(W_FindDialog, "*FindMtimeValue");
    retstr = XmTextFieldGetString(w_temp);
    strcat(command, retstr);
    strcat(command, " ");
  }

  /* ctime */

  w_temp = XtNameToWidget(W_FindDialog, "*FindAdvancedCtimeToggle");
  if(XmToggleButtonGetState(w_temp)) {
    strcat(command,"-ctime ");
    if(FindOptionCtime==2) strcat(command,"-");
    else if(FindOptionCtime==3) strcat(command,"+");
    w_temp = XtNameToWidget(W_FindDialog, "*FindCtimeValue");
    retstr = XmTextFieldGetString(w_temp);
    strcat(command, retstr);
    strcat(command, " ");
  }

  /* atime */

  w_temp = XtNameToWidget(W_FindDialog, "*FindAdvancedAtimeToggle");
  if(XmToggleButtonGetState(w_temp)) {
    strcat(command,"-atime ");
    if(FindOptionAtime==2) strcat(command,"-");
    else if(FindOptionAtime==3) strcat(command,"+");
    w_temp = XtNameToWidget(W_FindDialog, "*FindAtimeValue");
    retstr = XmTextFieldGetString(w_temp);
    strcat(command, retstr);
    strcat(command, " ");
  }
  
  /* perm */

  w_temp = XtNameToWidget(W_FindDialog, "*FindAdvancedPermToggle");
  if(XmToggleButtonGetState(w_temp)) {
    strcat(command,"-perm -");
    dummy=0;
    w_temp=XtNameToWidget(W_FindDialog, "*FindPermUserR");
    if(XmToggleButtonGetState(w_temp)) dummy+=4;
    w_temp=XtNameToWidget(W_FindDialog, "*FindPermUserW");
    if(XmToggleButtonGetState(w_temp)) dummy+=2;
    w_temp=XtNameToWidget(W_FindDialog, "*FindPermUserX");
    if(XmToggleButtonGetState(w_temp)) dummy+=1;
    sprintf(tempstr, "%1d", dummy);
    strcat(command, tempstr);
    dummy=0;
    w_temp=XtNameToWidget(W_FindDialog, "*FindPermGroupR");
    if(XmToggleButtonGetState(w_temp)) dummy+=4;
    w_temp=XtNameToWidget(W_FindDialog, "*FindPermGroupW");
    if(XmToggleButtonGetState(w_temp)) dummy+=2;
    w_temp=XtNameToWidget(W_FindDialog, "*FindPermGroupX");
    if(XmToggleButtonGetState(w_temp)) dummy+=1;
    sprintf(tempstr, "%1d", dummy);
    strcat(command, tempstr);
    dummy=0;
    w_temp=XtNameToWidget(W_FindDialog, "*FindPermOthersR");
    if(XmToggleButtonGetState(w_temp)) dummy+=4;
    w_temp=XtNameToWidget(W_FindDialog, "*FindPermOthersW");
    if(XmToggleButtonGetState(w_temp)) dummy+=2;
    w_temp=XtNameToWidget(W_FindDialog, "*FindPermOthersX");
    if(XmToggleButtonGetState(w_temp)) dummy+=1;
    sprintf(tempstr, "%1d", dummy);
    strcat(command, tempstr);
    strcat(command," ");
  }

  /* contents */

  w_temp = XtNameToWidget(W_FindDialog, "*FindAdvancedContentsToggle");
  if(XmToggleButtonGetState(w_temp)) {
    w_temp = XtNameToWidget(W_FindDialog, "*FindAdvancedContentsText");
    retstr = XmTextFieldGetString(w_temp);
    strcat(command, "-exec grep -l ");
    strcat(command, retstr);
    strcat(command, " {} ;");
  }
  /*  else 
      strcat(command, "-exec grep -l * {} ;");*/
  
  FindExecCommand(command);

  XtFree(command);

}

void FindStopButtonCB(Widget w_button, XtPointer app_data, XtPointer widget_data)
{
  if(FindChildExist)
    kill(FindChildPID, SIGKILL);
}

void FindCloseButtonCB(Widget w_button, XtPointer app_data, XtPointer widget_data)
{
  if(FindChildExist)
    kill(FindChildPID, SIGKILL);

  XtUnmanageChild(W_FindDialog);

}


/* toggle callbacks */

void FindBasicToggleCB(Widget w_button, XtPointer app_data, XtPointer widget_data)
{
  XmToggleButtonCallbackStruct *infos;
  Widget w_temp;

  infos = (XmToggleButtonCallbackStruct *) widget_data;

  if(infos->reason==XmCR_VALUE_CHANGED && infos->set==True) {
    XtUnmanageChild(W_FindAdvancedCriteria);
    w_temp = XtNameToWidget(W_FindDialog, "FindAdvancedToggle");
    XmToggleButtonSetState(w_temp, False, False);
    XtVaSetValues(W_FindBasicCriteria, XmNchildType, XmFRAME_WORKAREA_CHILD, NULL);
    XtManageChild(W_FindBasicCriteria);
  }
}
 
void FindAdvancedToggleCB(Widget w_button, XtPointer app_data, XtPointer widget_data)
{
  XmToggleButtonCallbackStruct *infos;
  Widget w_temp;

  infos = (XmToggleButtonCallbackStruct *) widget_data;

  if(infos->reason==XmCR_VALUE_CHANGED && infos->set==True) {
    XtUnmanageChild(W_FindBasicCriteria);
    w_temp = XtNameToWidget(W_FindDialog, "FindBasicToggle");
    XmToggleButtonSetState(w_temp, False, False);
    XtVaSetValues(W_FindAdvancedCriteria, XmNchildType, XmFRAME_WORKAREA_CHILD, NULL);
    XtManageChild(W_FindAdvancedCriteria);
  }
}
 
 
void FindNameToggleCB(Widget w_button, XtPointer app_data, XtPointer widget_data)
{
  XmToggleButtonCallbackStruct *infos;
  Widget w_temp;

  infos = (XmToggleButtonCallbackStruct *) widget_data;

  if(infos->reason==XmCR_VALUE_CHANGED)
    if (infos->set==True) {
      w_temp = XtNameToWidget(W_FindDialog, "*FindBasicNameText");
      XtVaSetValues(w_temp, XmNsensitive, True, NULL);
    }
    else if(infos->set==False) {
      w_temp = XtNameToWidget(W_FindDialog, "*FindBasicNameText");
      XtVaSetValues(w_temp, XmNsensitive, False, NULL);
    }
}

void FindTypeToggleCB(Widget w_button, XtPointer app_data, XtPointer widget_data)
{
  XmToggleButtonCallbackStruct *infos;
  Widget w_temp;

  infos = (XmToggleButtonCallbackStruct *) widget_data;

  if(infos->reason==XmCR_VALUE_CHANGED)
    if (infos->set==True) {
      w_temp = XtNameToWidget(W_FindDialog, "*FindBasicTypeButton");
      XtVaSetValues(w_temp, XmNsensitive, True, NULL);
    }
    else if(infos->set==False) {
      w_temp = XtNameToWidget(W_FindDialog, "*FindBasicTypeButton");
      XtVaSetValues(w_temp, XmNsensitive, False, NULL);
    }
}

void FindSizeToggleCB(Widget w_button, XtPointer app_data, XtPointer widget_data)
{
  XmToggleButtonCallbackStruct *infos;
  Widget w_temp;

  infos = (XmToggleButtonCallbackStruct *) widget_data;

  if(infos->reason==XmCR_VALUE_CHANGED)
    if (infos->set==True) {
      w_temp = XtNameToWidget(W_FindDialog, "*FindBasicSizeMenu1");
      XtVaSetValues(w_temp, XmNsensitive, True, NULL);
      XtVaSetValues(XmOptionButtonGadget(w_temp), XmNsensitive, True, NULL);
      w_temp = XtNameToWidget(W_FindDialog, "*FindBasicSizeMenu2");
      XtVaSetValues(w_temp, XmNsensitive, True, NULL);
      XtVaSetValues(XmOptionButtonGadget(w_temp), XmNsensitive, True, NULL);
      w_temp = XtNameToWidget(W_FindDialog, "*FindSizeValue");
      XtVaSetValues(w_temp, XmNsensitive, True, NULL);      
    }
    else if(infos->set==False) {
      w_temp = XtNameToWidget(W_FindDialog, "*FindBasicSizeMenu1");
      XtVaSetValues(w_temp, XmNsensitive, False, NULL);
      XtVaSetValues(XmOptionButtonGadget(w_temp), XmNsensitive, False, NULL);
      w_temp = XtNameToWidget(W_FindDialog, "*FindBasicSizeMenu2");
      XtVaSetValues(w_temp, XmNsensitive, False, NULL);
      XtVaSetValues(XmOptionButtonGadget(w_temp), XmNsensitive, False, NULL);
      w_temp = XtNameToWidget(W_FindDialog, "*FindSizeValue");
      XtVaSetValues(w_temp, XmNsensitive, False, NULL);      
    }
}

void FindUserToggleCB(Widget w_button, XtPointer app_data, XtPointer widget_data)
{
  XmToggleButtonCallbackStruct *infos;
  Widget w_temp;

  infos = (XmToggleButtonCallbackStruct *) widget_data;

  if(infos->reason==XmCR_VALUE_CHANGED)
    if (infos->set==True) {
      w_temp = XtNameToWidget(W_FindDialog, "*FindBasicUserText");
      XtVaSetValues(w_temp, XmNsensitive, True, NULL);
    }
    else if(infos->set==False) {
      w_temp = XtNameToWidget(W_FindDialog, "*FindBasicUserText");
      XtVaSetValues(w_temp, XmNsensitive, False, NULL);
    }
}

void FindGroupToggleCB(Widget w_button, XtPointer app_data, XtPointer widget_data)
{
  XmToggleButtonCallbackStruct *infos;
  Widget w_temp;

  infos = (XmToggleButtonCallbackStruct *) widget_data;

  if(infos->reason==XmCR_VALUE_CHANGED)
    if (infos->set==True) {
      w_temp = XtNameToWidget(W_FindDialog, "*FindBasicGroupText");
      XtVaSetValues(w_temp, XmNsensitive, True, NULL);
    }
    else if(infos->set==False) {
      w_temp = XtNameToWidget(W_FindDialog, "*FindBasicGroupText");
      XtVaSetValues(w_temp, XmNsensitive, False, NULL);
    }
}

void FindMtimeToggleCB(Widget w_button, XtPointer app_data, XtPointer widget_data)
{
  XmToggleButtonCallbackStruct *infos;
  Widget w_temp;

  infos = (XmToggleButtonCallbackStruct *) widget_data;

  if(infos->reason==XmCR_VALUE_CHANGED)
    if (infos->set==True) {
      w_temp = XtNameToWidget(W_FindDialog, "*FindAdvancedMtimeMenu");
      XtVaSetValues(w_temp, XmNsensitive, True, NULL);
      XtVaSetValues(XmOptionButtonGadget(w_temp), XmNsensitive, True, NULL);
      w_temp = XtNameToWidget(W_FindDialog, "*FindMtimeLabel");
      XtVaSetValues(w_temp, XmNsensitive, True, NULL);
      w_temp = XtNameToWidget(W_FindDialog, "*FindMtimeValue");
      XtVaSetValues(w_temp, XmNsensitive, True, NULL);      
    }
    else if(infos->set==False) {
      w_temp = XtNameToWidget(W_FindDialog, "*FindAdvancedMtimeMenu");
      XtVaSetValues(w_temp, XmNsensitive, False, NULL);
      XtVaSetValues(XmOptionButtonGadget(w_temp), XmNsensitive, False, NULL);
      w_temp = XtNameToWidget(W_FindDialog, "*FindMtimeLabel");
      XtVaSetValues(w_temp, XmNsensitive, False, NULL);
      w_temp = XtNameToWidget(W_FindDialog, "*FindMtimeValue");
      XtVaSetValues(w_temp, XmNsensitive, False, NULL);      
    }
}

void FindCtimeToggleCB(Widget w_button, XtPointer app_data, XtPointer widget_data)
{
  XmToggleButtonCallbackStruct *infos;
  Widget w_temp;

  infos = (XmToggleButtonCallbackStruct *) widget_data;

  if(infos->reason==XmCR_VALUE_CHANGED)
    if (infos->set==True) {
      w_temp = XtNameToWidget(W_FindDialog, "*FindAdvancedCtimeMenu");
      XtVaSetValues(w_temp, XmNsensitive, True, NULL);
      XtVaSetValues(XmOptionButtonGadget(w_temp), XmNsensitive, True, NULL);
      w_temp = XtNameToWidget(W_FindDialog, "*FindCtimeLabel");
      XtVaSetValues(w_temp, XmNsensitive, True, NULL);
      w_temp = XtNameToWidget(W_FindDialog, "*FindCtimeValue");
      XtVaSetValues(w_temp, XmNsensitive, True, NULL);      
    }
    else if(infos->set==False) {
      w_temp = XtNameToWidget(W_FindDialog, "*FindAdvancedCtimeMenu");
      XtVaSetValues(w_temp, XmNsensitive, False, NULL);
      XtVaSetValues(XmOptionButtonGadget(w_temp), XmNsensitive, False, NULL);
      w_temp = XtNameToWidget(W_FindDialog, "*FindCtimeLabel");
      XtVaSetValues(w_temp, XmNsensitive, False, NULL);
      w_temp = XtNameToWidget(W_FindDialog, "*FindCtimeValue");
      XtVaSetValues(w_temp, XmNsensitive, False, NULL);      
    }
}

void FindAtimeToggleCB(Widget w_button, XtPointer app_data, XtPointer widget_data)
{
  XmToggleButtonCallbackStruct *infos;
  Widget w_temp;

  infos = (XmToggleButtonCallbackStruct *) widget_data;

  if(infos->reason==XmCR_VALUE_CHANGED)
    if (infos->set==True) {
      w_temp = XtNameToWidget(W_FindDialog, "*FindAdvancedAtimeMenu");
      XtVaSetValues(w_temp, XmNsensitive, True, NULL);
      XtVaSetValues(XmOptionButtonGadget(w_temp), XmNsensitive, True, NULL);
      w_temp = XtNameToWidget(W_FindDialog, "*FindAtimeLabel");
      XtVaSetValues(w_temp, XmNsensitive, True, NULL);
      w_temp = XtNameToWidget(W_FindDialog, "*FindAtimeValue");
      XtVaSetValues(w_temp, XmNsensitive, True, NULL);      
    }
    else if(infos->set==False) {
      w_temp = XtNameToWidget(W_FindDialog, "*FindAdvancedAtimeMenu");
      XtVaSetValues(w_temp, XmNsensitive, False, NULL);
      XtVaSetValues(XmOptionButtonGadget(w_temp), XmNsensitive, False, NULL);
      w_temp = XtNameToWidget(W_FindDialog, "*FindAtimeLabel");
      XtVaSetValues(w_temp, XmNsensitive, False, NULL);
      w_temp = XtNameToWidget(W_FindDialog, "*FindAtimeValue");
      XtVaSetValues(w_temp, XmNsensitive, False, NULL);      
    }
}

void FindPermToggleCB(Widget w_button, XtPointer app_data, XtPointer widget_data)
{
  XmToggleButtonCallbackStruct *infos;
  Widget w_temp;

  infos = (XmToggleButtonCallbackStruct *) widget_data;

  if(infos->reason==XmCR_VALUE_CHANGED)
    if (infos->set==True) {
      w_temp = XtNameToWidget(W_FindDialog, "*FindAdvancedPermUser");
      XtVaSetValues(w_temp, XmNsensitive, True, NULL);
      w_temp = XtNameToWidget(W_FindDialog, "*FindAdvancedPermGroup");
      XtVaSetValues(w_temp, XmNsensitive, True, NULL);
      w_temp = XtNameToWidget(W_FindDialog, "*FindAdvancedPermOthers");
      XtVaSetValues(w_temp, XmNsensitive, True, NULL);      
    }
    else if(infos->set==False) {
      w_temp = XtNameToWidget(W_FindDialog, "*FindAdvancedPermUser");
      XtVaSetValues(w_temp, XmNsensitive, False, NULL);
      w_temp = XtNameToWidget(W_FindDialog, "*FindAdvancedPermGroup");
      XtVaSetValues(w_temp, XmNsensitive, False, NULL);
      w_temp = XtNameToWidget(W_FindDialog, "*FindAdvancedPermOthers");
      XtVaSetValues(w_temp, XmNsensitive, False, NULL);      
    }
}

void FindContentsToggleCB(Widget w_button, XtPointer app_data, XtPointer widget_data)
{
  XmToggleButtonCallbackStruct *infos;
  Widget w_temp;

  infos = (XmToggleButtonCallbackStruct *) widget_data;

  if(infos->reason==XmCR_VALUE_CHANGED)
    if (infos->set==True) {
      w_temp = XtNameToWidget(W_FindDialog, "*FindAdvancedContentsText");
      XtVaSetValues(w_temp, XmNsensitive, True, NULL);
    }
    else if(infos->set==False) {
      w_temp = XtNameToWidget(W_FindDialog, "*FindAdvancedContentsText");
      XtVaSetValues(w_temp, XmNsensitive, False, NULL);
    }
}

/* popup menu callbacks */

void FindTypeChangedCB(Widget w_button, XtPointer app_data, XtPointer widget_data)
{
  Widget w_temp;
  char message[64];
  char add[32];
  XmString xmessage;
  int count;

  count=0;
  strcpy(message,"Choose (now:");

  strcpy(add," ");

  w_temp=XtNameToWidget(W_FindDialog, "*FindTypeRegular");
  if(XmToggleButtonGetState(w_temp)) {
    strcat(add,"f ");
    count++;
  }

  w_temp=XtNameToWidget(W_FindDialog, "*FindTypeDir");
  if(XmToggleButtonGetState(w_temp)) {
    strcat(add,"d ");
    count++;
  }

  w_temp=XtNameToWidget(W_FindDialog, "*FindTypeSLink");
  if(XmToggleButtonGetState(w_temp)) {
    strcat(add,"l ");
    count++;
  }

  w_temp=XtNameToWidget(W_FindDialog, "*FindTypeBlock");
  if(XmToggleButtonGetState(w_temp)) {
    strcat(add,"b ");
    count++;
  }

  w_temp=XtNameToWidget(W_FindDialog, "*FindTypeChar");
  if(XmToggleButtonGetState(w_temp)) {
    strcat(add,"c ");
    count++;
  }

  w_temp=XtNameToWidget(W_FindDialog, "*FindTypeFifo");
  if(XmToggleButtonGetState(w_temp)) {
    strcat(add,"p ");
    count++;
  }

  w_temp=XtNameToWidget(W_FindDialog, "*FindTypeSocket");
  if(XmToggleButtonGetState(w_temp)) {
    strcat(add,"s ");
    count++;
  }

  if(count==0 || count==7)
    strcat(message," all )");
  else {
    strcat(message, add);
    strcat(message,")");
  }

  xmessage=XmStringCreateSimple(message);
  w_temp=XtNameToWidget(W_FindDialog, "*FindBasicTypeButton");
  XtVaSetValues(w_temp, XmNlabelString, xmessage, NULL);
  XmStringFree(xmessage);
}

void FindPermUserChangedCB(Widget w_button, XtPointer app_data, XtPointer widget_data)
{
  Widget w_temp;
  char message[64];
  XmString xmessage;

  strcpy(message,"User (now: ");

  w_temp=XtNameToWidget(W_FindDialog, "*FindPermUserR");
  if(XmToggleButtonGetState(w_temp))
    strcat(message,"r");
  else strcat(message,"-");

  w_temp=XtNameToWidget(W_FindDialog, "*FindPermUserW");
  if(XmToggleButtonGetState(w_temp))
    strcat(message,"w");
  else strcat(message,"-");

  w_temp=XtNameToWidget(W_FindDialog, "*FindPermUserX");
  if(XmToggleButtonGetState(w_temp))
    strcat(message,"x");
  else strcat(message,"-");

  strcat(message," )");

  xmessage=XmStringCreateSimple(message);
  w_temp=XtNameToWidget(W_FindDialog, "*FindAdvancedPermUser");
  XtVaSetValues(w_temp, XmNlabelString, xmessage, NULL);
  XmStringFree(xmessage);

}

void FindPermGroupChangedCB(Widget w_button, XtPointer app_data, XtPointer widget_data)
{
  Widget w_temp;
  char message[64];
  XmString xmessage;

  strcpy(message,"Group (now: ");

  w_temp=XtNameToWidget(W_FindDialog, "*FindPermGroupR");
  if(XmToggleButtonGetState(w_temp))
    strcat(message,"r");
  else strcat(message,"-");

  w_temp=XtNameToWidget(W_FindDialog, "*FindPermGroupW");
  if(XmToggleButtonGetState(w_temp))
    strcat(message,"w");
  else strcat(message,"-");

  w_temp=XtNameToWidget(W_FindDialog, "*FindPermGroupX");
  if(XmToggleButtonGetState(w_temp))
    strcat(message,"x");
  else strcat(message,"-");

  strcat(message," )");

  xmessage=XmStringCreateSimple(message);
  w_temp=XtNameToWidget(W_FindDialog, "*FindAdvancedPermGroup");
  XtVaSetValues(w_temp, XmNlabelString, xmessage, NULL);
  XmStringFree(xmessage);

}

void FindPermOthersChangedCB(Widget w_button, XtPointer app_data, XtPointer widget_data)
{
  Widget w_temp;
  char message[64];
  XmString xmessage;

  strcpy(message,"Others (now: ");

  w_temp=XtNameToWidget(W_FindDialog, "*FindPermOthersR");
  if(XmToggleButtonGetState(w_temp))
    strcat(message,"r");
  else strcat(message,"-");

  w_temp=XtNameToWidget(W_FindDialog, "*FindPermOthersW");
  if(XmToggleButtonGetState(w_temp))
    strcat(message,"w");
  else strcat(message,"-");

  w_temp=XtNameToWidget(W_FindDialog, "*FindPermOthersX");
  if(XmToggleButtonGetState(w_temp))
    strcat(message,"x");
  else strcat(message,"-");

  strcat(message," )");

  xmessage=XmStringCreateSimple(message);
  w_temp=XtNameToWidget(W_FindDialog, "*FindAdvancedPermOthers");
  XtVaSetValues(w_temp, XmNlabelString, xmessage, NULL);
  XmStringFree(xmessage);

}

void FindActionCB(Widget w_item, XtPointer app_data, XtPointer widget_data)
{
  int i;
  XmListCallbackStruct *infos;
  char *destfile;
  char *destdir;
  char *destname;
 
  infos = (XmListCallbackStruct *) widget_data;

  if(infos->reason==XmCR_DEFAULT_ACTION) {
    XmStringGetLtoR(infos->item, XmFONTLIST_DEFAULT_TAG, &destfile);

    /* get the destination dir */

    destdir=XtMalloc(strlen(destfile)+1);
    strcpy(destdir, destfile);
    i=strlen(destdir);
    while(destdir[i]!='/' && i>0) {
      destdir[i]=(char)0;
      i--;
    }

    /* go to the dir if possible */
    
    if(GoToDirRefreshRoutine(destdir)==-1) {
      XtFree(destdir);
      XtFree(destfile);
      return;
    }

    /* now, try to select the file */

    destname=XtMalloc(strlen(destfile)+1);
    i=0;
    while(strchr(&destfile[i], '/')!=NULL) i++;

    strcpy(destname, &destfile[i]);

    SelectFilterCommandRoutine(destname);
      
    XtFree(destname);
    XtFree(destdir);
    XtFree(destfile);
  }    

}

/* Dialog creation */

void FindInitDialog(void)
{
  Widget w_frame1, w_frame2, w_separator, w_button;
  Widget w_label, w_text, w_menu, w_pane, w_menu2, w_pane2, w_menu3, w_pane3 ;
  Widget w_rowcol;
  Widget w_temp;
  Widget w_bar;


  W_FindDialog=XmCreateFormDialog(W_MainWindow,"FindDialog", NULL,0);

  XtVaSetValues(W_FindDialog, 
		XmNdialogStyle, XmDIALOG_MODELESS,
		XmNautoUnmanage, False,
		NULL);

  w_rowcol = XtVaCreateManagedWidget("FindDirRC", xmRowColumnWidgetClass, W_FindDialog,
				     XmNtopAttachment, XmATTACH_FORM,
				     XmNleftAttachment, XmATTACH_FORM,
				     XmNrightAttachment, XmATTACH_FORM,
				     XmNorientation, XmHORIZONTAL,
				     NULL);

  w_label = XtVaCreateManagedWidget("FindDirLabel", xmLabelWidgetClass, w_rowcol,
				    NULL);

  w_text = XtVaCreateManagedWidget("FindDirText", xmTextFieldWidgetClass, w_rowcol,
				   XmNresizeWidth, True,
				   NULL);

  w_separator = XtVaCreateManagedWidget("FindSeparator1", xmSeparatorWidgetClass, W_FindDialog,
					XmNtopAttachment, XmATTACH_WIDGET,
					XmNtopWidget, w_rowcol,
					XmNleftAttachment, XmATTACH_FORM,
					XmNrightAttachment, XmATTACH_FORM,
					NULL);

  /* find criteria */

  w_button = XtVaCreateManagedWidget("FindBasicToggle", xmToggleButtonWidgetClass, W_FindDialog,
				     XmNtopAttachment, XmATTACH_WIDGET,
				     XmNtopWidget, w_separator,
				     XmNleftAttachment, XmATTACH_FORM,
				     XmNset, True,
				     XmNfillOnSelect, True,
				     XmNindicatorOn, XmINDICATOR_NONE,
				     XmNshadowThickness, 2,
				     NULL);

  XtAddCallback(w_button, XmNvalueChangedCallback, FindBasicToggleCB, NULL);


  w_button = XtVaCreateManagedWidget("FindAdvancedToggle", xmToggleButtonWidgetClass, W_FindDialog,
				     XmNtopAttachment, XmATTACH_WIDGET,
				     XmNtopWidget, w_separator,
				     XmNleftAttachment, XmATTACH_WIDGET,
				     XmNleftWidget, w_button,
				     XmNfillOnSelect, True,
				     XmNindicatorOn, XmINDICATOR_NONE,
				     XmNshadowThickness, 2,
				     NULL);

  XtAddCallback(w_button, XmNvalueChangedCallback, FindAdvancedToggleCB, NULL);


  w_frame1 = XtVaCreateManagedWidget("FindCriteriaFrame", xmFrameWidgetClass, W_FindDialog,
				     XmNtopAttachment, XmATTACH_WIDGET,
				     XmNtopWidget, w_button,
				     XmNleftAttachment, XmATTACH_FORM,
				     XmNrightAttachment, XmATTACH_FORM,
				     NULL);


  /* basic criteria */

  W_FindBasicCriteria = XtVaCreateManagedWidget("FindBasicCriteria", xmFormWidgetClass, w_frame1,
					      XmNchildType, XmFRAME_WORKAREA_CHILD,
					      NULL);

  /* name */

  w_rowcol = XtVaCreateManagedWidget("FindBasicNameRC", xmRowColumnWidgetClass, W_FindBasicCriteria,
				     XmNtopAttachment, XmATTACH_FORM,
				     XmNleftAttachment, XmATTACH_FORM,
				     XmNrightAttachment, XmATTACH_FORM,
				     XmNorientation, XmHORIZONTAL,
				     NULL);


  w_button = XtVaCreateManagedWidget("FindBasicNameToggle", xmToggleButtonWidgetClass, w_rowcol,
				     XmNset, False,
				     NULL);

  XtAddCallback(w_button, XmNvalueChangedCallback, FindNameToggleCB, NULL);


  w_text = XtVaCreateManagedWidget("FindBasicNameText", xmTextFieldWidgetClass, w_rowcol,
				   XmNresizeWidth, True,
				   XmNsensitive, False,
				   NULL);

  /* type */

  w_rowcol = XtVaCreateManagedWidget("FindBasicTypeRC", xmRowColumnWidgetClass, W_FindBasicCriteria,
				     XmNtopAttachment, XmATTACH_WIDGET,
				     XmNtopWidget, w_rowcol,
				     XmNleftAttachment, XmATTACH_FORM,
				     XmNrightAttachment, XmATTACH_FORM,
				     XmNorientation, XmHORIZONTAL,
				     NULL);


  w_button = XtVaCreateManagedWidget("FindBasicTypeToggle", xmToggleButtonWidgetClass, w_rowcol,
				     XmNset, False,
				     NULL);

  XtAddCallback(w_button, XmNvalueChangedCallback, FindTypeToggleCB, NULL);


  w_bar = XmCreateMenuBar(w_rowcol, "FindBasicTypeBar", NULL, 0);

  w_pane = XmCreatePulldownMenu( w_bar, "FindBasicTypePane", NULL, 0);

  XtAddCallback(w_pane, XmNentryCallback, FindTypeChangedCB, NULL);

  w_menu = XtVaCreateManagedWidget("FindBasicTypeButton", xmCascadeButtonWidgetClass, w_bar, 
				   XmNsensitive, False,
				   XmNsubMenuId, w_pane,
				   NULL);

  XtVaCreateManagedWidget( "FindTypeRegular", xmToggleButtonWidgetClass, w_pane, NULL);
  XtVaCreateManagedWidget( "FindTypeDir", xmToggleButtonWidgetClass, w_pane, NULL);
  XtVaCreateManagedWidget( "FindTypeSLink", xmToggleButtonWidgetClass, w_pane, NULL);
  XtVaCreateManagedWidget( "FindTypeBlock", xmToggleButtonWidgetClass, w_pane, NULL);
  XtVaCreateManagedWidget( "FindTypeChar", xmToggleButtonWidgetClass, w_pane, NULL);
  XtVaCreateManagedWidget( "FindTypeFifo", xmToggleButtonWidgetClass, w_pane, NULL);
  XtVaCreateManagedWidget( "FindTypeSocket", xmToggleButtonWidgetClass, w_pane, NULL);

  XtManageChild(w_bar);

  /* size */

  w_rowcol = XtVaCreateManagedWidget("FindBasicSizeRC", xmRowColumnWidgetClass, W_FindBasicCriteria,
				     XmNtopAttachment, XmATTACH_WIDGET,
				     XmNtopWidget, w_rowcol,
				     XmNleftAttachment, XmATTACH_FORM,
				     XmNrightAttachment, XmATTACH_FORM,
				     XmNorientation, XmHORIZONTAL,
 				     NULL);

   w_button = XtVaCreateManagedWidget("FindBasicSizeToggle", xmToggleButtonWidgetClass, w_rowcol,
				     XmNset, False,
				     NULL);

  XtAddCallback(w_button, XmNvalueChangedCallback, FindSizeToggleCB, NULL);

  w_menu2 = XmCreateOptionMenu( w_rowcol, "FindBasicSizeMenu1", NULL, 0);

  w_temp = XmOptionButtonGadget(w_menu2);
  XtUnmanageChild(w_temp);

  w_pane2 = XmCreatePulldownMenu( w_rowcol, "FindBasicSizePane1", NULL, 0);
  XtVaSetValues(w_menu2, XmNsubMenuId, w_pane2, XmNsensitive, FALSE, NULL);

  XtAddCallback(w_pane2, XmNentryCallback, SizeOption1CB, NULL);

  XtVaCreateManagedWidget( "FindSizeExact", xmPushButtonWidgetClass, w_pane2, NULL);
  XtVaCreateManagedWidget( "FindSizeLess", xmPushButtonWidgetClass, w_pane2, NULL);
  XtVaCreateManagedWidget( "FindSizeMore", xmPushButtonWidgetClass, w_pane2, NULL);

  XtManageChild(w_menu2);

  XtManageChild(w_temp);
  XtVaSetValues(w_temp, XmNsensitive, False, NULL);

  w_text = XtVaCreateManagedWidget("FindSizeValue", xmTextFieldWidgetClass, w_rowcol,
				   XmNresizeWidth, True,
				   XmNsensitive, False,
				   NULL);


  w_menu3 = XmCreateOptionMenu( w_rowcol, "FindBasicSizeMenu2", NULL, 0);

  w_temp = XmOptionButtonGadget(w_menu3);
  XtUnmanageChild(w_temp);

  w_pane3 = XmCreatePulldownMenu( w_rowcol, "FindBasicSizePane2", NULL, 0);
  XtVaSetValues(w_menu3, XmNsubMenuId, w_pane3, XmNsensitive, False, NULL);

  XtAddCallback(w_pane3, XmNentryCallback, SizeOption2CB, NULL);

  XtVaCreateManagedWidget( "FindSizeBytes", xmPushButtonWidgetClass, w_pane3, NULL);
  XtVaCreateManagedWidget( "FindSizeKilos", xmPushButtonWidgetClass, w_pane3, NULL);
  XtVaCreateManagedWidget( "FindSizeBlocks", xmPushButtonWidgetClass, w_pane3, NULL);
  XtVaCreateManagedWidget( "FindSizeWords", xmPushButtonWidgetClass, w_pane3, NULL);

  XtManageChild(w_menu3);
  XtManageChild(w_temp);
  XtVaSetValues(w_temp, XmNsensitive, False, NULL);


  /* user */

  w_rowcol = XtVaCreateManagedWidget("FindBasicUserRC", xmRowColumnWidgetClass, W_FindBasicCriteria,
				     XmNtopAttachment, XmATTACH_WIDGET,
				     XmNtopWidget, w_rowcol,
				     XmNleftAttachment, XmATTACH_FORM,
				     XmNrightAttachment, XmATTACH_FORM,
				     XmNorientation, XmHORIZONTAL,
				     NULL);


  w_button = XtVaCreateManagedWidget("FindBasicUserToggle", xmToggleButtonWidgetClass, w_rowcol,
				     XmNset, False,
				     NULL);


  XtAddCallback(w_button, XmNvalueChangedCallback, FindUserToggleCB, NULL);


  w_text = XtVaCreateManagedWidget("FindBasicUserText", xmTextFieldWidgetClass, w_rowcol,
				   XmNresizeWidth, True,
				   XmNsensitive, False,
				   NULL);

   /* group */

  w_rowcol = XtVaCreateManagedWidget("FindBasicGroupRC", xmRowColumnWidgetClass, W_FindBasicCriteria,
				     XmNtopAttachment, XmATTACH_WIDGET,
				     XmNtopWidget, w_rowcol,
				     XmNleftAttachment, XmATTACH_FORM,
				     XmNrightAttachment, XmATTACH_FORM,
				     XmNorientation, XmHORIZONTAL,
				     NULL);

  w_button = XtVaCreateManagedWidget("FindBasicGroupToggle", xmToggleButtonWidgetClass, w_rowcol,
				     XmNset, False,
				     NULL);

  XtAddCallback(w_button, XmNvalueChangedCallback, FindGroupToggleCB, NULL);


  w_text = XtVaCreateManagedWidget("FindBasicGroupText", xmTextFieldWidgetClass, w_rowcol,
				   XmNresizeWidth, True,
				   XmNsensitive, False,
				   NULL);
   /* one filesystem */

  w_button = XtVaCreateManagedWidget("FindBasicOneFsToggle", xmToggleButtonWidgetClass, W_FindBasicCriteria,
				     XmNtopAttachment, XmATTACH_WIDGET,
				     XmNtopWidget, w_rowcol,
				     XmNleftAttachment, XmATTACH_FORM,
				     XmNset, False,
				     NULL);

  /* advanced criteria */

  W_FindAdvancedCriteria = XtVaCreateWidget("FindAdvancedCriteria", xmFormWidgetClass, w_frame1,
					      XmNchildType, XmFRAME_WORKAREA_CHILD,
					      NULL);


  /* mtime */

  w_rowcol = XtVaCreateManagedWidget("FindAdvancedMtimeRC", xmRowColumnWidgetClass, W_FindAdvancedCriteria,
				     XmNtopAttachment, XmATTACH_FORM,
				     XmNleftAttachment, XmATTACH_FORM,
				     XmNrightAttachment, XmATTACH_FORM,
				     XmNorientation, XmHORIZONTAL,
				     NULL);


  w_button = XtVaCreateManagedWidget("FindAdvancedMtimeToggle", xmToggleButtonWidgetClass, w_rowcol,
				     XmNset, False,
				     NULL);

  XtAddCallback(w_button, XmNvalueChangedCallback, FindMtimeToggleCB, NULL);

  w_menu = XmCreateOptionMenu( w_rowcol, "FindAdvancedMtimeMenu", NULL, 0);

  w_temp = XmOptionButtonGadget(w_menu);
  XtUnmanageChild(w_temp);

  w_pane = XmCreatePulldownMenu( w_rowcol, "FindAdvancedMtimePane", NULL, 0);
  XtVaSetValues(w_menu, XmNsubMenuId, w_pane, XmNsensitive, False, NULL);

  XtAddCallback(w_pane, XmNentryCallback, MtimeOptionCB, NULL);

  XtVaCreateManagedWidget( "FindMtimeExact", xmPushButtonWidgetClass, w_pane, NULL);
  XtVaCreateManagedWidget( "FindMtimeLess", xmPushButtonWidgetClass, w_pane, NULL);
  XtVaCreateManagedWidget( "FindMtimeMore", xmPushButtonWidgetClass, w_pane, NULL);

  XtManageChild(w_menu);

  XtManageChild(w_temp);
  XtVaSetValues(w_temp, XmNsensitive, False, NULL);

  w_text = XtVaCreateManagedWidget("FindMtimeValue", xmTextFieldWidgetClass, w_rowcol,
				   XmNresizeWidth, True,
				   XmNsensitive, False,
				   NULL);


  w_label= XtVaCreateManagedWidget( "FindMtimeLabel", xmLabelWidgetClass, w_rowcol,
				    XmNsensitive, False,
				    NULL);


  /* ctime */
  
  w_rowcol = XtVaCreateManagedWidget("FindAdvancedCtimeRC", xmRowColumnWidgetClass, W_FindAdvancedCriteria,
				     XmNtopAttachment, XmATTACH_WIDGET,
				     XmNtopWidget, w_rowcol,
				     XmNleftAttachment, XmATTACH_FORM,
				     XmNrightAttachment, XmATTACH_FORM,
				     XmNorientation, XmHORIZONTAL,
				     NULL);


  w_button = XtVaCreateManagedWidget("FindAdvancedCtimeToggle", xmToggleButtonWidgetClass, w_rowcol,
				     XmNset, False,
				     NULL);

  XtAddCallback(w_button, XmNvalueChangedCallback, FindCtimeToggleCB, NULL);

  w_menu = XmCreateOptionMenu( w_rowcol, "FindAdvancedCtimeMenu", NULL, 0);

  w_temp = XmOptionButtonGadget(w_menu);
  XtUnmanageChild(w_temp);

  w_pane = XmCreatePulldownMenu( w_rowcol, "FindAdvancedCtimePane", NULL, 0);
  XtVaSetValues(w_menu, XmNsubMenuId, w_pane, XmNsensitive, False, NULL);

  XtAddCallback(w_pane, XmNentryCallback, CtimeOptionCB, NULL);

  XtVaCreateManagedWidget( "FindCtimeExact", xmPushButtonWidgetClass, w_pane, NULL);
  XtVaCreateManagedWidget( "FindCtimeLess", xmPushButtonWidgetClass, w_pane, NULL);
  XtVaCreateManagedWidget( "FindCtimeMore", xmPushButtonWidgetClass, w_pane, NULL);

  XtManageChild(w_menu);

  XtManageChild(w_temp);
  XtVaSetValues(w_temp, XmNsensitive, False, NULL);

  w_text = XtVaCreateManagedWidget("FindCtimeValue", xmTextFieldWidgetClass, w_rowcol,
				   XmNresizeWidth, True,
				   XmNsensitive, False,
				   NULL);


  w_label= XtVaCreateManagedWidget( "FindCtimeLabel", xmLabelWidgetClass, w_rowcol,
				    XmNsensitive, False,
				    NULL);


  
  /* atime */

  w_rowcol = XtVaCreateManagedWidget("FindAdvancedAtimeRC", xmRowColumnWidgetClass, W_FindAdvancedCriteria,
				     XmNtopAttachment, XmATTACH_WIDGET,
				     XmNtopWidget, w_rowcol,
				     XmNleftAttachment, XmATTACH_FORM,
				     XmNrightAttachment, XmATTACH_FORM,
				     XmNorientation, XmHORIZONTAL,
				     NULL);


  w_button = XtVaCreateManagedWidget("FindAdvancedAtimeToggle", xmToggleButtonWidgetClass, w_rowcol,
				     XmNset, False,
				     NULL);

  XtAddCallback(w_button, XmNvalueChangedCallback, FindAtimeToggleCB, NULL);

  w_menu = XmCreateOptionMenu( w_rowcol, "FindAdvancedAtimeMenu", NULL, 0);

  w_temp = XmOptionButtonGadget(w_menu);
  XtUnmanageChild(w_temp);

  w_pane = XmCreatePulldownMenu( w_rowcol, "FindAdvancedAtimePane", NULL, 0);
  XtVaSetValues(w_menu, XmNsubMenuId, w_pane, XmNsensitive, False, NULL);

  XtAddCallback(w_pane, XmNentryCallback, AtimeOptionCB, NULL);

  XtVaCreateManagedWidget( "FindAtimeExact", xmPushButtonWidgetClass, w_pane, NULL);
  XtVaCreateManagedWidget( "FindAtimeLess", xmPushButtonWidgetClass, w_pane, NULL);
  XtVaCreateManagedWidget( "FindAtimeMore", xmPushButtonWidgetClass, w_pane, NULL);

  XtManageChild(w_menu);

  XtManageChild(w_temp);
  XtVaSetValues(w_temp, XmNsensitive, False, NULL);

  w_text = XtVaCreateManagedWidget("FindAtimeValue", xmTextFieldWidgetClass, w_rowcol,
				   XmNresizeWidth, True,
				   XmNsensitive, False,
				   NULL);


  w_label= XtVaCreateManagedWidget( "FindAtimeLabel", xmLabelWidgetClass, w_rowcol,
				    XmNsensitive, False,
				    NULL);


  
  /* permissions */

  w_rowcol = XtVaCreateManagedWidget("FindAdvancedPermRC", xmRowColumnWidgetClass, W_FindAdvancedCriteria,
				     XmNtopAttachment, XmATTACH_WIDGET,
				     XmNtopWidget, w_rowcol,
				     XmNleftAttachment, XmATTACH_FORM,
				     XmNrightAttachment, XmATTACH_FORM,
				     XmNorientation, XmHORIZONTAL,
				     NULL);


  w_button = XtVaCreateManagedWidget("FindAdvancedPermToggle", xmToggleButtonWidgetClass, w_rowcol,
				     XmNset, False,
				     NULL);

  XtAddCallback(w_button, XmNvalueChangedCallback, FindPermToggleCB, NULL);

  w_bar = XmCreateMenuBar(w_rowcol, "FindAdvancedPermBar1", NULL, 0);

  w_pane = XmCreatePulldownMenu( w_bar, "FindAdvancedPermPane1", NULL, 0);

   XtAddCallback(w_pane, XmNentryCallback, FindPermUserChangedCB, NULL);

  w_menu = XtVaCreateManagedWidget("FindAdvancedPermUser", xmCascadeButtonWidgetClass, w_bar, 
				   XmNsubMenuId, w_pane,
				   XmNsensitive, False,
				   NULL);

  XtVaCreateManagedWidget( "FindPermUserR", xmToggleButtonWidgetClass, w_pane, NULL);
  XtVaCreateManagedWidget( "FindPermUserW", xmToggleButtonWidgetClass, w_pane, NULL);
  XtVaCreateManagedWidget( "FindPermUserX", xmToggleButtonWidgetClass, w_pane, NULL);

  XtManageChild(w_bar);

  w_bar = XmCreateMenuBar(w_rowcol, "FindAdvancedPermBar2", NULL, 0);

  w_pane2 = XmCreatePulldownMenu( w_bar, "FindAdvancedPermPane2", NULL, 0);

  XtAddCallback(w_pane2, XmNentryCallback, FindPermGroupChangedCB, NULL);


  w_menu2 = XtVaCreateManagedWidget("FindAdvancedPermGroup", xmCascadeButtonWidgetClass, w_bar, 
				    XmNsubMenuId, w_pane2,
				    XmNsensitive, False,
				    NULL);

  XtVaCreateManagedWidget( "FindPermGroupR", xmToggleButtonWidgetClass, w_pane2, NULL);
  XtVaCreateManagedWidget( "FindPermGroupW", xmToggleButtonWidgetClass, w_pane2, NULL);
  XtVaCreateManagedWidget( "FindPermGroupX", xmToggleButtonWidgetClass, w_pane2, NULL);

  XtManageChild(w_bar);

  w_bar = XmCreateMenuBar(w_rowcol, "FindAdvancedPermBar2", NULL, 0);

  w_pane3 = XmCreatePulldownMenu( w_bar, "FindAdvancedPermPane3", NULL, 0);

  XtAddCallback(w_pane3, XmNentryCallback, FindPermOthersChangedCB, NULL);
   
  w_menu3 = XtVaCreateManagedWidget("FindAdvancedPermOthers", xmCascadeButtonWidgetClass, w_bar, 
				    XmNsubMenuId, w_pane3,
				    XmNsensitive, False,
				    NULL);

  XtVaCreateManagedWidget( "FindPermOthersR", xmToggleButtonWidgetClass, w_pane3, NULL);
  XtVaCreateManagedWidget( "FindPermOthersW", xmToggleButtonWidgetClass, w_pane3, NULL);
  XtVaCreateManagedWidget( "FindPermOthersX", xmToggleButtonWidgetClass, w_pane3, NULL);

  XtManageChild(w_bar);

  /* contents */

  w_rowcol = XtVaCreateManagedWidget("FindAdvancedContentsRC", xmRowColumnWidgetClass, W_FindAdvancedCriteria,
				     XmNtopAttachment, XmATTACH_WIDGET,
				     XmNtopWidget, w_rowcol,
				     XmNleftAttachment, XmATTACH_FORM,
				     XmNrightAttachment, XmATTACH_FORM,
				     XmNorientation, XmHORIZONTAL,
				     NULL);


  w_button = XtVaCreateManagedWidget("FindAdvancedContentsToggle", xmToggleButtonWidgetClass, w_rowcol,
				     XmNset, False,
				     NULL);

  XtAddCallback(w_button, XmNvalueChangedCallback, FindContentsToggleCB, NULL);

  w_text = XtVaCreateManagedWidget("FindAdvancedContentsText", xmTextFieldWidgetClass, w_rowcol,
				   XmNresizeWidth, True,
				   XmNsensitive, False,
				   NULL);


  /* command buttons */
					     
  w_button = XtVaCreateManagedWidget("FindHelpButton", xmPushButtonWidgetClass, W_FindDialog,
				     XmNrightAttachment, XmATTACH_FORM,
				     XmNbottomAttachment, XmATTACH_FORM,
				     NULL);

  w_button = XtVaCreateManagedWidget("FindCloseButton", xmPushButtonWidgetClass, W_FindDialog,
				     XmNrightAttachment, XmATTACH_WIDGET,
				     XmNrightWidget, w_button,
				     XmNbottomAttachment, XmATTACH_FORM,
				     NULL);

  XtAddCallback(w_button, XmNactivateCallback, FindCloseButtonCB, NULL);

  w_button = XtVaCreateManagedWidget("FindStopButton", xmPushButtonWidgetClass, W_FindDialog,
				     XmNrightAttachment, XmATTACH_WIDGET,
				     XmNrightWidget, w_button,
				     XmNbottomAttachment, XmATTACH_FORM,
				     XmNsensitive, False,
				     NULL);

  XtAddCallback(w_button, XmNactivateCallback, FindStopButtonCB, NULL);

  w_button = XtVaCreateManagedWidget("FindExecuteButton", xmPushButtonWidgetClass, W_FindDialog,
				     XmNrightAttachment, XmATTACH_WIDGET,
				     XmNrightWidget, w_button,
				     XmNbottomAttachment, XmATTACH_FORM,
				     XmNleftAttachment, XmATTACH_FORM,
				     NULL);

  XtVaSetValues(W_FindDialog, XmNdefaultButton, w_button, NULL);

  XtAddCallback(w_button, XmNactivateCallback, FindExecButtonCB, NULL);

  w_separator = XtVaCreateManagedWidget("FindSeparator", xmSeparatorWidgetClass, W_FindDialog,
					XmNleftAttachment, XmATTACH_FORM,
					XmNrightAttachment, XmATTACH_FORM,
					XmNbottomAttachment, XmATTACH_WIDGET,
					XmNbottomWidget, w_button,
					NULL);

  /* result frame */
  
  w_frame2 = XtVaCreateManagedWidget("FindFrame2", xmFrameWidgetClass, W_FindDialog,
				     XmNtopAttachment, XmATTACH_WIDGET,
				     XmNtopWidget, w_frame1,
				     XmNleftAttachment, XmATTACH_FORM,
				     XmNrightAttachment, XmATTACH_FORM,
				     XmNbottomAttachment, XmATTACH_WIDGET,
				     XmNbottomWidget, w_separator,
				     NULL);
  
  w_label = XtVaCreateManagedWidget("FindLabel2", xmLabelWidgetClass, w_frame2,
				    XmNchildType, XmFRAME_TITLE_CHILD,
				    NULL);


  W_FindList = XmCreateScrolledList(w_frame2, "FindList", NULL, 0);

  XtManageChild(W_FindList);

  XtVaSetValues(W_FindList, 
		XmNvisibleItemCount, 10, 
		XmNselectionPolicy, XmEXTENDED_SELECT,
		NULL);
  
  
  XtVaSetValues(W_FindBasicCriteria, XmNchildType, XmFRAME_WORKAREA_CHILD, NULL);

  XtAddCallback(W_FindList, XmNdefaultActionCallback, FindActionCB, NULL);


  #ifdef OFFIX_DND

  DND_FindListInitialize(W_FindList);

  #endif

}


void FindRoutine(char *fromdir)
{
  Widget w_temp;

  /* create the dialog if necessary */

  if(W_FindDialog==NULL)
    FindInitDialog();
  
  XtManageChild(W_FindDialog);

  /* if there is a dir */

  if(fromdir!=NULL) {
    w_temp=XtNameToWidget(W_FindDialog, "*FindDirText");
    XmTextFieldSetString(w_temp, fromdir);
  }  else {
    w_temp=XtNameToWidget(W_FindDialog, "*FindDirText");
    XmTextFieldSetString(w_temp, "/");
  }
}



